/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package experSwingJfreeChart;

import java.io.PrintWriter;

/**
 *
 * @author pchr
 */
public class sdofOscillator {
    private double stif;
    private double mass;
    private double dampRatio;
    
    private double[] time=null;
    private double[] fext=null;
    private double[] disp;
    private double[] velc;
    private double[] accl;
    
    private double u0;
    private double v0;
    
    private double timeStep;
    private double beta;
    private double gamma;
    
    private int steps;
    
    // constructor
    public sdofOscillator(){
    };
    
    // methods
    public void initOscillator(ESDOF theESDOF){
        this.stif = theESDOF.getK();
        this.mass = theESDOF.getM();
        this.dampRatio = theESDOF.getZ();
        
        this.u0=theESDOF.getu0();
        this.v0=theESDOF.getv0();
        
        this.timeStep=theESDOF.getDt();
        this.gamma=theESDOF.getGamma();
        this.beta=theESDOF.getBeta();
        
        this.time=new double[1]; time[0]=0.;
        this.fext=new double[1]; fext[0]=0.;
        this.disp=new double[1]; disp[0]=0.;
        this.velc=new double[1]; velc[0]=0.;
        this.accl=new double[1]; accl[0]=0.;
        
    }
    
    public double[] getTime(){
        return this.time;
    }
    
    public double[] getFext(){
        return this.fext;
    }
    
    public void setTime(double[] vals){
        this.time=new double[vals.length];
        for(int i=0; i<vals.length; i++){
            this.time[i]=vals[i];
        }
    }
    
    public void setFext(double[] vals){
        this.fext=new double[vals.length];
        for(int i=0; i<vals.length; i++){
            this.fext[i]=vals[i];
        }
    }
    
    public void run(){
        double[] w= new double[8];
        w[0]=1./(beta*timeStep*timeStep);   w[1]=gamma/(beta*timeStep); w[2]=1.0/(beta*timeStep);
        w[3]=1.0/(2*beta)-1.;               w[4]=gamma/beta-1.;         w[5]=timeStep*(gamma/beta-2.)/2.0;
        w[6]=timeStep*(1.-gamma);           w[7]=timeStep*gamma;
        double damp=2.0*dampRatio*mass*Math.sqrt(stif/mass)/100.;
        double keff=stif+w[0]*mass+w[1]*damp;
        double feff;
        // chect fext, time
        if(fext!=null&&time!=null){
            steps=Math.min(fext.length,time.length);
            disp = new double[steps]; velc = new double[steps]; accl = new double[steps];
            disp[0]=u0; velc[0]=v0; accl[0]=(fext[0]-damp*v0-stif*u0)/mass;
        }else{steps=0;}
        
        for(int i=1; i<steps; i++){
            feff=fext[i]+mass*(w[0]*disp[i-1]+w[2]*velc[i-1]+w[3]*accl[i-1])
                     +damp*(w[1]*disp[i-1]+w[4]*velc[i-1]+w[5]*accl[i-1]);
            disp[i]=feff/keff;
            accl[i]=w[0]*(disp[i]-disp[i-1])-w[2]*velc[i-1]-w[3]*accl[i-1];
            velc[i]=velc[i-1]+w[6]*accl[i-1]+w[7]*accl[i];
        }
    }
    
    public int getNSteps(){
        return this.steps;
    }
    
    public double[] getDisp(){
        return this.disp;
    }
    
    public double[] getVelc(){
        return this.velc;
    }
    
    public double[] getAccl(){
        return this.accl;
    }
    
    public double[] getHistory(int ind){
        double History[] = new double[steps];
        switch(ind){
            case 0 : History=this.getTime(); break ;
            case 1 : History=this.getFext(); break ;
            case 2 : History=this.getDisp(); break ;
            case 3 : History=this.getVelc(); break ;
            case 4 : History=this.getAccl(); break ;
            default:  ; break ;
        }
        return History;
    }
    
    public void createResults(PrintWriter SaveResults){
        SaveResults.print("t");SaveResults.print(" ");
        SaveResults.print("fetx");SaveResults.print(" ");
        SaveResults.print("d");SaveResults.print(" ");
        SaveResults.print("v");SaveResults.print(" ");
        SaveResults.print("a");SaveResults.println();
        int len=this.time.length;
        for(int i=0; i<len ; i++){
            SaveResults.print(this.time[i]);SaveResults.print(" ");
            SaveResults.print(this.fext[i]);SaveResults.print(" ");
            SaveResults.print(this.disp[i]);SaveResults.print(" ");
            SaveResults.print(this.velc[i]);SaveResults.print(" ");
            SaveResults.print(this.accl[i]);SaveResults.println();
        }
        SaveResults.close();
    }

}
